/* See accompanying file, LICENSE, for license information */

/* This is an attemp at sandboxing the XMLHttpRequest. In previous version
 * cookies got mixed with the user session making it very hard, if not impossible,
 * to be logged on as admin while having the extension installed, since the
 * extension would log you out, then log you on as the ordinary user.
 */

// This is the variable that will hold the session cookie for the extension.

if(typeof BUBBAMON === 'undefined') {
    var BUBBAMON = {};
}

BUBBAMON.HtmlConnection = function(url, request, callback, origin, timeout)
{
    var req = new XMLHttpRequest();

    if(request) {
        req.open("POST",url,true);
        req.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
    } else {
        req.open("GET",url,true);
    }
    req.setRequestHeader('Accept', '*/*');

    new BUBBAMON.Sandbox(req, timeout);

    req.onreadystatechange = function(){
        if (req.readyState == 4) {
            if (callback) {
                callback(req.responseText, origin);
            }
        }
    };

    req.send(request);	
}


BUBBAMON.Sandbox = function(req, timeout) {
    this.mChannel = req.channel;

    var observerService = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);	

    observerService.addObserver(this, "http-on-modify-request", false);
    observerService.addObserver(this, "http-on-examine-response", false);
    this.mTimer = new BUBBAMON.TimeOut((timeout - 1000), this);
}

BUBBAMON.Sandbox.prototype.destroy = function(caller) {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);	
    observerService.removeObserver(caller, "http-on-modify-request");
    observerService.removeObserver(caller, "http-on-examine-response");

    delete(caller.mChannel);
    caller.mTimer.stop();
    delete(caller.mTimer);
    delete(caller.mObserverService);
}

BUBBAMON.Sandbox.prototype.observe = function(aSubject, aTopic, aData) {
    switch(aTopic) {
    case "http-on-modify-request":
        // Replace session cookie in out going message with our
        // cookie.
        if (aSubject == this.mChannel) {
            var httpChannel = aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
            httpChannel.setRequestHeader("Cookie", BUBBAMON.sessionCookie, false)
        }
        break;
    case "http-on-examine-response":
        if(aSubject == this.mChannel) {
            var httpChannel = aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);

            try {
                if(BUBBAMON.sessionCookie == "") {
                    // Extract cookie if we haven't set it already
                    BUBBAMON.sessionCookie = httpChannel.getResponseHeader("Set-Cookie");
                }
            }
            catch(e){}

            // Now remove it so we don't mix the cookies.
            httpChannel.setResponseHeader("Set-Cookie", "", false);
        }
        break;
    }
}

// we are faking an XPCOM interface, so we need to implement QI
BUBBAMON.Sandbox.prototype.QueryInterface = function(aIID){
    if (aIID.equals(Components.interfaces.nsISupports) ||
            aIID.equals(Components.interfaces.nsIObserver))
        return this;

    throw Components.results.NS_NOINTERFACE;
}

BUBBAMON.TimeOut = function(timeout, origin) {
    this.mOrigin = origin;
    this.mTimer = Components.classes["@mozilla.org/timer;1"].
        createInstance(Components.interfaces.nsITimer);
    this.mObserverService = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    this.mObserverService.addObserver(this, "xpcom-shutdown", false);

    this.mTimer.initWithCallback(this, timeout, this.mTimer.TYPE_ONE_SHOT);
}

BUBBAMON.TimeOut.prototype.notify = function(aTimer) {
    this.mOrigin.destroy(this.mOrigin);
    this.stop();
}

BUBBAMON.TimeOut.prototype.stop = function() {
    if(this.mTimer) {
        this.mTimer.cancel();
        this.mTimer = null;
        this.mOrigin = null;
        this.mObserverService.removeObserver(this, "xpcom-shutdown");		
        this.mObserverService = null;
    }
}

BUBBAMON.TimeOut.prototype.observe = function(aSubject, aTopic, aData) {
    if(aTopic == "xpcom-shutdown") {
        this.stop();
    }
}

// we are faking an XPCOM interface, so we need to implement QI
BUBBAMON.TimeOut.prototype.QueryInterface = function(aIID) {
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsIObserver) ||
        aIID.equals(Components.interfaces.nsITimerCallback))
        return this;

    throw Components.results.NS_NOINTERFACE;
}

