/* See accompanying file, LICENSE, for license information */

if(typeof BUBBAMON === 'undefined') {
    var BUBBAMON = {};
}

BUBBAMON.BubbaTwo = function() {
   this.addr = BUBBAMON.prefs.address + "/admin/";
   this.timer = null;
   this.timeout_value = BUBBAMON.prefs.interval * 2;
   this.timeout_timer = null;
   this.enabled = false;
   this.nativeJSON = Components.classes["@mozilla.org/dom/json;1"]
      .createInstance(Components.interfaces.nsIJSON);

}

BUBBAMON.BubbaTwo.prototype.start_timeout = function(observer) {
   this.timeout_timer = window.setTimeout(function(){ observer.notify(); }, this.timeout_value);
}

BUBBAMON.BubbaTwo.prototype.stop_timeout = function() {
   // Stop timeout timer
   if(this.timeout_timer != null) {
      clearTimeout(this.timeout_timer);
      this.timeout_timer = null;
   }
}

BUBBAMON.BubbaTwo.prototype.logOn = function(msg) {
   this.enabled = false;
   if(msg == undefined) {
      msg = "Connecting...";
   }
  
   BUBBAMON.setStatus(msg);

   var observer = new BUBBAMON.BubbaTwo.LogonRetry(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "login/index/json", "submit=Login" +
         "&username=" +BUBBAMON.prefs.user +
         "&password=" +BUBBAMON.prefs.passwd, 
         this.verifyLogOn, this, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaTwo.prototype.logOff = function() {
   this.stop_timeout();
   this.enabled = false;
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }

   BUBBAMON.HtmlConnection(this.addr + "logout", "logout=yes", null, null, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaTwo.prototype.verifyLogOn = function(response, origin) {
   var obj;
   var logged_on = false;

   if(response == "") {
      return;
   }

   origin.stop_timeout();

   try {
      obj = origin.nativeJSON.decode(response);

      if(obj.authfail == false) {
         BUBBAMON.setStatus("Connected");
         origin.refresh();
         logged_on = true;
      }
   }
   catch(e) {
      // Fallback, look for "Invalid user"
      if(response.search(/Invalid user/) == -1) {
         BUBBAMON.setStatus("Connected");
         origin.refresh();
         logged_on = true;
      }
   }
   if (!logged_on && (BUBBAMON.prefs.nagCounter == 0)) {
      // Annoying nag counter.
      BUBBAMON.setStatus("Invalid user/passwd");
      alert("Invalid user/password combination.\n" + 
            "Verify you are using the right identity.");
      BUBBAMON.prefs.nagCounter++;			
   }
}
   
BUBBAMON.BubbaTwo.prototype.refresh = function() {
   this.enabled = true;
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }
   BUBBAMON.HtmlConnection(this.addr + "downloads/dolist/json", null, this.parseDLList, this, BUBBAMON.prefs.interval);
   BUBBAMON.HtmlConnection(this.addr + "stat/index/json", null, this.parseGeneral, this, BUBBAMON.prefs.interval);

   var observer = new BUBBAMON.BubbaTwo.RefreshRetry(this);
   this.start_timeout(observer);
}

BUBBAMON.BubbaTwo.prototype.parseDLList = function(html, origin) {
   if(!origin.enabled) {
      return;
   }
   var obj;
   
   try {
      obj = origin.nativeJSON.decode(html);
   }
   catch(e) {
      return;
   }

   if(obj) {
      BUBBAMON.updateDownloads( obj, 0, 0);
   }
}

BUBBAMON.BubbaTwo.prototype.parseGeneral = function(html, origin) {
   if(!origin.enabled) {
      return;
   }

   // Stop timeout timer
   origin.stop_timeout();
   // Start refresh timer
   origin.timer = window.setTimeout(function(){ origin.refresh(); },BUBBAMON.prefs.interval);

   // Fix to remove the PHP error
   var re = /\{.*\}/;
   var html = re.exec(html);
   try {
      var obj = origin.nativeJSON.decode(html);
      BUBBAMON.updateGenerals(obj);
   }
   catch(e) {
      origin.enabled = false;
      origin.logOn("Reconnecting...");
   }
}

BUBBAMON.BubbaTwo.prototype.pause = function() {
   this.enabled = false;
   this.stop_timeout();
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }
}

BUBBAMON.BubbaTwo.prototype.addDownload = function(url, uuid) {
   this.pause();
   
   // Start timeout timer
   var observer = new BUBBAMON.BubbaTwo.Resume(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "downloads/add/json", "add_download=Add" + 
         "&uuid=" + uuid + 
         "&url=" + url, this.restart, this, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaTwo.prototype.cancelDownload = function(src, uuid) {
   this.pause();

   // Start timeout timer
   var observer = new BUBBAMON.BubbaTwo.Resume(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "downloads/remove/json", "do=Cancel" +
         "&url=" + src +
         "&uuid=" + uuid, this.restart, this, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaTwo.prototype.restart = function(response, origin)
{
   origin.stop_timeout();
   origin.refresh();
}

BUBBAMON.BubbaTwo.Resume = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      this.origin.stop_timeout();
      this.origin.refresh();
   }
}

BUBBAMON.BubbaTwo.LogonRetry = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      if(this.origin.enabled) {
         this.origin.logOn("Retry logon...");
      }
   }
}

BUBBAMON.BubbaTwo.RefreshRetry = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      this.origin.refresh();
   }
}


